"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.IvrTester = void 0;
const TwilioCallServer_1 = require("./testing/TwilioCallServer");
const PluginManager_1 = require("./plugins/PluginManager");
const populateDefaults_1 = require("./configuration/populateDefaults");
const TwilioCaller_1 = require("./call/TwilioCaller");
const IteratingTestAssigner_1 = require("./testing/IteratingTestAssigner");
const MediaStreamRecorder_1 = require("./call/recording/MediaStreamRecorder");
const TestExecutor_1 = require("./testing/TestExecutor");
const AudioPlaybackCaller_1 = require("./call/AudioPlaybackCaller");
const consoleUserInterface_1 = require("./testing/ui/consoleUserInterface");
const StopTestRunnerWhenTestsComplete_1 = require("./testing/StopTestRunnerWhenTestsComplete");
const callConnectedTimeout_1 = require("./testing/callConnectedTimeout");
const TranscriptRecorder_1 = require("./call/recording/TranscriptRecorder");
function createTestRunnerManager() {
    let callback = undefined;
    let stopped = false;
    let stoppedDueToFailure = false;
    return {
        setOnStopCallback(cb) {
            callback = cb;
            if (stopped) {
                callback(stoppedDueToFailure);
            }
        },
        testRunner: {
            stop(failure = false) {
                stopped = true;
                stoppedDueToFailure = failure;
                if (callback) {
                    callback(stoppedDueToFailure);
                }
            },
        },
    };
}
function createPluginManager(config) {
    const userInterface = consoleUserInterface_1.consoleUserInterface();
    return new PluginManager_1.PluginManager([
        new StopTestRunnerWhenTestsComplete_1.StopTestRunnerWhenTestsComplete(),
        userInterface,
        callConnectedTimeout_1.callConnectedTimeout(config, userInterface),
        MediaStreamRecorder_1.mediaStreamRecorderPlugin(config),
        TranscriptRecorder_1.transcriptRecorderPlugin(config),
    ]);
}
class IvrTester {
    constructor(configuration) {
        this.running = false;
        this.config = populateDefaults_1.populateDefaults(configuration);
        this.pluginManager = createPluginManager(this.config);
    }
    async run(call, ivrTest) {
        if (this.running) {
            throw new Error("Instance of IvrTester can only run a single test suite");
        }
        this.running = true;
        const tests = Array.isArray(ivrTest) ? ivrTest : [ivrTest];
        await this.preflightChecks();
        const callServer = new TwilioCallServer_1.TwilioCallServer(this.config.dtmfGenerator, new IteratingTestAssigner_1.IteratingTestAssigner(tests), TestExecutor_1.testExecutor(this.config.transcriber));
        const caller = Buffer.isBuffer(call)
            ? new AudioPlaybackCaller_1.AudioPlaybackCaller()
            : new TwilioCaller_1.TwilioCaller(this.config.twilioClient);
        const testRunnerManager = createTestRunnerManager();
        this.pluginManager.initialise(testRunnerManager.testRunner);
        const serverUrl = await callServer.listen(this.config.localServerPort);
        this.pluginManager.serverListening(callServer);
        const calls = Promise.all(tests.map(() => caller
            .call(call, this.config.publicServerUrl || serverUrl)
            .then((callRequested) => this.pluginManager.callRequested(callRequested, tests.length))
            .catch((error) => {
            this.pluginManager.callRequestErrored(new Error(error));
            throw error;
        })));
        return new Promise((resolve, reject) => {
            calls
                .then(() => {
                callServer.on("stopped", reject);
                callServer.on("error", reject);
                testRunnerManager.setOnStopCallback((failure) => {
                    callServer.off("stopped", reject);
                    callServer.off("error", reject);
                    callServer
                        .stop()
                        .catch((err) => err && console.error(err))
                        .finally(() => {
                        if (failure) {
                            reject();
                        }
                        else {
                            resolve();
                        }
                    });
                });
            })
                .catch((error) => {
                callServer.off("stopped", reject);
                callServer.off("error", reject);
                callServer
                    .stop()
                    .catch((err) => err && console.error(err))
                    .finally(() => reject(error));
            });
        });
    }
    async preflightChecks() {
        const checkResults = await this.config.transcriber.checkCanRun();
        if (checkResults.canRun === false) {
            throw new Error(`Error with the transcriber you've chosen:\n${checkResults.reason}`);
        }
    }
}
exports.IvrTester = IvrTester;
